package com.smartadserver.android.sasnewsreader;

import android.util.Log;
import android.view.View;

import com.smartadserver.android.library.model.SASNativeAdElement;
import com.smartadserver.android.library.model.SASNativeAdManager;
import com.smartadserver.android.library.model.SASNativeAdManager.NativeAdResponseHandler;

/**
 * This class extends the base class ListElement by adding internal native ad support : its properties
 * will be filled by the native ad returned by the ad call, if any.
 */
public class AdElement extends ListElement {

	SASNativeAdManager mNativeAdManager;
	SASNativeAdElement nativeAd;
	
	View viewPendingRegistration = null;
	View[] clickableViewsPending = null;
	boolean adRequested = false;
	
	/**
	 * Returns whether the underlying native ad was fetched already.
	 */
	public boolean isAdReady() {
		return nativeAd != null;
	}

	/**
	 * Register the specified View with the underlying native ad, if any
	 */
	public void registerView(View v) {
		registerView(v,null);
	}

	/**
	 * Register the specified View and clickableViews with the underlying native ad, if any.
	 * If the native ad is null, stores these views to be registered as soon as the native ad is ready
	 */
	public synchronized void registerView(View v, View[] clickableViews) {
		if (nativeAd != null) {
			try {
				if (clickableViews == null) {
					nativeAd.registerView(v);
				} else {
					nativeAd.registerView(v, clickableViews);
				}
			} catch (Exception e) {
				// happens if already registered, just discard..
				Log.d("AdElement","NativeAd: " + e.getMessage());
			}
		} else {
			// register view when ad is ready
			viewPendingRegistration = v;
			clickableViewsPending = clickableViews;
		}
	}


	/**
	 * Unregister the specified View with the underlying native ad (or cancels pending registration if
	 * the native ad is not ready yet)
	 */
	public synchronized void unregisterView(View v) {
		if (nativeAd != null) {
			nativeAd.unregisterView(v);
		}
		viewPendingRegistration = null;
		clickableViewsPending = null;
	}


	/**
	 * 
	 * @param nativeAdManager
	 */
	public AdElement(SASNativeAdManager nativeAdManager) {
		super();
		this.mNativeAdManager = nativeAdManager;
	}
	
	
	/**
	 * 
	 * @param delegateAdResponseHandler
	 */
	public void requestAd(final NativeAdResponseHandler delegateAdResponseHandler) {

		// ad can be requested only once
		if (adRequested) {
			return;
		} else {
			adRequested = true;
		}
		
		NativeAdResponseHandler adResponseHandler = new NativeAdResponseHandler() {
			
			@Override
			public void nativeAdLoadingCompleted(SASNativeAdElement nativeAd) {

				synchronized(AdElement.this) {
					AdElement.this.nativeAd = nativeAd;

					nativeAd.setClickHandler(new SASNativeAdElement.ClickHandler() {
						@Override
						public boolean handleClick(String clickUrl, SASNativeAdElement nativeAdElement) {

							return true;
						}
					});

					if (viewPendingRegistration != null) {
						try {
							if (clickableViewsPending == null) {
								nativeAd.registerView(viewPendingRegistration);
							} else {
								nativeAd.registerView(viewPendingRegistration, clickableViewsPending);
							}
						} catch (Exception e) {
							// happens if already registered, just discard..
							Log.d("AdElement","NativeAd: " + e.getMessage());
						}
						viewPendingRegistration = null;
						clickableViewsPending = null;
					}
				}
				
				// update this ad element with ad content
				AdElement.this.title = nativeAd.getTitle();
				AdElement.this.details = nativeAd.getBody();
				SASNativeAdElement.ImageElement icon = nativeAd.getIcon();
				if (icon != null) {
					AdElement.this.thumbnailUrl = nativeAd.getIcon().getUrl();
				}

				AdElement.this.clickUrl = nativeAd.getClickUrl();
				AdElement.this.actionLabel = nativeAd.getCalltoAction();
				// notify delegate ad response handler that ad loading succeeded
				delegateAdResponseHandler.nativeAdLoadingCompleted(nativeAd);
			}
			
			@Override
			public void nativeAdLoadingFailed(Exception e) {
				delegateAdResponseHandler.nativeAdLoadingFailed(e);
			}

		};
		mNativeAdManager.requestNativeAd(adResponseHandler,5000);
	}
	
}
