package com.smartadserver.android.sasnewsreader;

import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;

import com.smartadserver.android.library.model.SASNativeAdElement;
import com.smartadserver.android.library.model.SASNativeAdManager.NativeAdResponseHandler;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Handler;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewGroup.LayoutParams;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;


/**
 * The adapter class responsible for creating RecyclerView.ViewHolder instances for different cells.
 */
class ListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

	public static String TAG = ListAdapter.class.getSimpleName();

	// ArrayList containing content items for cells
	ArrayList<ListElement> itemList;

	// Handler needed for task execution on main thread
	Handler uiHandler;
	Context context;

	// Bitmap to use as a placeholder for cells :
	// - without any image available or
	// - whose image is not downloaded yet
	private Bitmap transparentBitmap;

	SASNativeAdElement.ClickHandler clickHandler;

	/**
	 * ViewHolder subclass for all cells (content as well as native ads)
	 */
	static class ListItemViewHolder extends RecyclerView.ViewHolder {

		public TextView titleTextView;
		public TextView detailTextView;
		public ImageView imageView=null;
		public Button actionButton = null;
		public LinearLayout adPanel = null;

		public AdElement adElement; // optional, only for ad cells

		public ListItemViewHolder(View itemView) {
			super(itemView);

			titleTextView = (TextView) itemView.findViewById(R.id.title);
			detailTextView = (TextView) itemView.findViewById(R.id.details);
			imageView = (ImageView) itemView.findViewById(R.id.img);
			actionButton = (Button)itemView.findViewById(R.id.actionButton);
			adPanel = (LinearLayout)itemView.findViewById(R.id.adPanel);
		}
	}

	/**
	 * Constructs a ListAdapter based on the specified element list.
	 * @param newsItemList the ArrayList containing data items to be displayed by the RecyclerView
	 * @param context the Context needed to start a new activity for clicks
	 */
	ListAdapter(ArrayList<ListElement> newsItemList, final Context context) {
		itemList = newsItemList;
		this.context = context;
		uiHandler = new Handler(context.getMainLooper());

		// Bitmap used when no image is available
		transparentBitmap = Bitmap.createBitmap(1,1,Bitmap.Config.RGB_565);

		clickHandler = new SASNativeAdElement.ClickHandler() {
			@Override
			public boolean handleClick(final String clickUrl, SASNativeAdElement nativeAdElement) {
				uiHandler.post(new Runnable() {
					@Override
					public void run() {
						Toast.makeText(context,"intercept click to " + clickUrl, Toast.LENGTH_LONG).show();
					}
				});
				return false;
			}
		};

	}

	@Override
	/**
	 * Implement abstract methods of RecyclerView.Adapter
	 */
	public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
		View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.list_item, parent, false);
		return new ListItemViewHolder(v);
	}

	/**
	 * Opens the proper Activity to view the specified URL
	 */
	private void openUrl(String url) {
		Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
		// start activity
		context.startActivity(intent);
	}

	/**
	 * Implement abstract methods of RecyclerView.Adapter
	 */
	@Override
	public void onViewRecycled(RecyclerView.ViewHolder holder) {
		super.onViewRecycled(holder);

		// cast to the actual ListItemViewHolder class
		ListItemViewHolder listItemViewHolder = (ListItemViewHolder)holder;

		/////////////////////////////////////////////////////////
		// MANDATORY : for Ad element, unregister the View of this ViewHolder with the AdElement
		/////////////////////////////////////////////////////////
		if (listItemViewHolder.adElement != null) {
			listItemViewHolder.adElement.unregisterView(holder.itemView);
			listItemViewHolder.adElement = null;
		}
	}

	@Override
	/**
	 * Implement abstract methods of RecyclerView.Adapter
	 */
	public void onBindViewHolder(RecyclerView.ViewHolder viewHolder, final int position) {

		// cast to the actual ListItemViewHolder class
		ListItemViewHolder holder = (ListItemViewHolder)viewHolder;
		// get the ListElement for the current position in the data model.
		final ListElement currentListElement= itemList.get(position);

		// is this ListElement an ad ?
		boolean isAd = currentListElement instanceof AdElement;

		// update values in the cell View according to ListElement properties
		holder.titleTextView.setText(currentListElement.title);
		holder.detailTextView.setText(currentListElement.details);

		if (isAd) {
			// if element is an ad, add yellow background
			holder.itemView.setBackgroundColor(0xFFFFFFDD);
			// and show a panel with an action button for click (open, download,...)
			holder.adPanel.setVisibility(View.VISIBLE);
			// set the lable of the Button according to the callToAction for this native ad
			holder.actionButton.setText(currentListElement.actionLabel);

			// cast to the actual AdElement class
			AdElement adElement = (AdElement)currentListElement;

			//request a native ad, if not done already
			adElement.requestAd(new NativeAdResponseHandler() {

				@Override
				public void nativeAdLoadingFailed(Exception e) {
					Log.d(TAG,"adLoadingFailed for ad at position " + position);
				}

				@Override
				public void nativeAdLoadingCompleted(final SASNativeAdElement nativeAdElement) {
					Log.d(TAG, "adLoadingCompleted for ad at position " + position);

					nativeAdElement.setClickHandler(clickHandler);
					// when native ad loading is completed, update the ad cell with native ad info
					uiHandler.post(new Runnable() {
						public void run() {
							notifyItemChanged(position);
						}
					});
				}
			});

			// store AdElement needed for unregistering when the view of this holder is recycled
			holder.adElement = adElement;

			/////////////////////////////////////////////////////////
			// MANDATORY : register the bound View with the native ad
			/////////////////////////////////////////////////////////
			adElement.registerView(viewHolder.itemView,new View[] {holder.actionButton});

			// hide or show View contents whether the ad is ready or not
			if (adElement.isAdReady()) {
				viewHolder.itemView.findViewById(R.id.subLinearLayout).setVisibility(View.VISIBLE);
			} else {
				viewHolder.itemView.findViewById(R.id.subLinearLayout).setVisibility(View.GONE);
			}

		} else {
			// this is not an ad but a regular content cell
			// hide ad button panel
			((ListItemViewHolder) viewHolder).adPanel.setVisibility(View.GONE);

			// set cell background to white
			final View itemView = viewHolder.itemView;
			itemView.setBackgroundColor(Color.WHITE);

			// show all view contents and set a click listener on the cell to display the associated URL
			itemView.findViewById(R.id.subLinearLayout).setVisibility(View.VISIBLE);
			viewHolder.itemView.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					openUrl(currentListElement.clickUrl);
				}
			});
		}

		// now update cell icon, and launch a background download if necessary
		// get width and height of image icon
		final int vWidth = holder.imageView.getLayoutParams().width;
		final int vHeight = holder.imageView.getLayoutParams().height;

		// need to fetch poster for this item if null
		if (currentListElement.posterBitmap == null && currentListElement.thumbnailUrl != null) {

			// set temporary transparent drawable to mark this item as being downloaded
			currentListElement.posterBitmap = transparentBitmap;

			// create download thread to asynchronously fetch image
			Thread imageLoader = new Thread() {
				public void run() {

					// fetch image drawable on this thread
					final Drawable drawable = imageFromUrl(currentListElement.thumbnailUrl);
					if (drawable != null) {
						// scale this image to match list item icon size
						Bitmap bitmap = ((BitmapDrawable)drawable).getBitmap();
						int bWidth = bitmap.getWidth();
						int bHeight = bitmap.getHeight();
						double resizeRatio = Math.min(vWidth / (double)bWidth, vHeight / (double)bHeight);
						Bitmap scaledBitmap = Bitmap.createScaledBitmap(bitmap, (int)(bWidth * resizeRatio), (int)(bHeight * resizeRatio), true);

						// now store scaled bitmap in video element
						currentListElement.posterBitmap = scaledBitmap;

						// ...then refresh image view in list
						uiHandler.post(new Runnable() {
							public void run() {
								notifyItemChanged(position);
							}
						});
					}
				}
			};
			// start image download thread
			imageLoader.start();
		}
		// now set it on the cell icon
		if (currentListElement.posterBitmap != null) {
			holder.imageView.setImageBitmap(currentListElement.posterBitmap);
		}
	}

	@Override
	/**
	 * Implement abstract methods of RecyclerView.Adapter
	 */
	public int getItemCount() {
		return itemList.size();
	}

	/**
	 * Convenience method to create a Drawable from a url.
	 */
	static Drawable imageFromUrl(String url)
	{
		try
		{
			InputStream is = (InputStream) new URL(url).getContent();
			Drawable d = Drawable.createFromStream(is, "src name");
			return d;
		}catch (Exception e) {
			System.out.println("Exc="+e);
			return null;
		}
	}

}