package com.smartadserver.android.sample;

import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import com.smartadserver.android.library.SASBannerView;
import com.smartadserver.android.library.SASInterstitialView;
import com.smartadserver.android.library.model.SASAdElement;
import com.smartadserver.android.library.ui.SASAdView;
import com.smartadserver.android.library.ui.SASRotatingImageLoader;

/**
 * Simple activity featuring a banner ad and an interstitial ad.
 */
public class SampleActivity extends Activity {

    /*****************************************
     * Members declarations
     *****************************************/
	// Banner view (as declared in the main.xml layout file, in res/layout)
	SASBannerView mBannerView;
	// Interstitial view (this view is not part of any xml layout file)
	SASInterstitialView mInterstitialView;
	
	// handler classes to be notified of ad loading outcome
	SASAdView.AdResponseHandler bannerResponseHandler;
	SASAdView.AdResponseHandler interstitialResponseHandler;
	
	// Buttons declared in main.xml
	Button mRefreshBannerButton;
	Button mDisplayInterstitialButton;

	
    /**
     * performs Activity initialization after creation
     */
	@Override
	public void onCreate(Bundle savedInstanceState) {
        
		super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        
        /*****************************************
         * now perform Ad related code here
         *****************************************/
	    // enable output to Android Logcat (optional)
        SASAdView.enableLogging();

	    // initialize both SASBannerView and SASInterstitialView
	    initBannerView();
	    initInterstitialView();

	    // create buttons to manually refresh interstitial or banner
        mDisplayInterstitialButton = (Button)this.findViewById(R.id.displayInterstitial);
	    mDisplayInterstitialButton.setOnClickListener(new View.OnClickListener() {
		    public void onClick(View v) {
		    	loadInterstitialAd();
		    }
	    });
	    
	    mRefreshBannerButton = (Button)this.findViewById(R.id.refreshBanner);
	    mRefreshBannerButton.setOnClickListener(new View.OnClickListener() {
		    public void onClick(View v) {
		    	loadBannerAd();
		    }
	    });
	    
	    
	    // load interstitial ad now
	    loadBannerAd();
	    
	    // display interstitial ad now, right at activity start 
	    loadInterstitialAd();

    }
	
	/**
	 * initialize the SASBannerView instance of this Activity
	 */
	private void initBannerView() {
		// fetch the SASBannerView inflated from the main.xml layout file
        mBannerView = (SASBannerView)this.findViewById(R.id.banner);

	    // Add a loader view on the banner. This view covers the banner placement, to indicate progress, whenever the banner is loading an ad.  
	    View loader = new SASRotatingImageLoader(this);
	    loader.setBackgroundColor(0x66000000);
	    mBannerView.setLoaderView(loader);
	    
	    // instantiate the response handler used when loading an ad on the banner
	    bannerResponseHandler = new SASAdView.AdResponseHandler() {
			public void adLoadingCompleted(SASAdElement adElement) {
				Log.i("Sample", "Banner loading completed");
			}

			public void adLoadingFailed(Exception e) {
				Log.i("Sample", "Banner loading failed: " + e.getMessage());
			}
		};
	}
	
	/**
	 * Loads an ad on the banner
	 */
	private void loadBannerAd() {
		// load banner ad with appropriate parameters (siteID,pageID,formatID,master,targeting,adResponseHandler)
		mBannerView.loadAd(35176, "(news_activity)", 15140, true, "", bannerResponseHandler);
	}


	/**
	 * initialize the SASInterstitialView instance of this Activity
	 */
	private void initInterstitialView() {
		
		// create SASInterstitialView instance
		mInterstitialView = new SASInterstitialView(this);

		// Add a loader view on the interstitial view. This view is displayed fullscreen, to indicate progress,
		// whenever the interstitial is loading an ad.
		View loader = new SASRotatingImageLoader(this);
		loader.setBackgroundColor(Color.WHITE);
		mInterstitialView.setLoaderView(loader);

		// add a state change listener on the SASInterstitialView instance to monitor MRAID states changes.
		// Useful for instance to perform some actions as soon as the interstitial disappears.
		mInterstitialView.addStateChangeListener(new SASAdView.OnStateChangeListener() {
			public void onStateChanged(SASAdView.StateChangeEvent stateChangeEvent) {
				switch(stateChangeEvent.getType()) {
					case SASAdView.StateChangeEvent.VIEW_DEFAULT:
						// the MRAID Ad View is in default state
						Log.i("Sample", "Interstitial MRAID state : DEFAULT");
						break;
					case SASAdView.StateChangeEvent.VIEW_EXPANDED:
						// the MRAID Ad View is in expanded state
						Log.i("Sample", "Interstitial MRAID state : EXPANDED");
						break;
					case SASAdView.StateChangeEvent.VIEW_HIDDEN:
						// the MRAID Ad View is in hidden state
						Log.i("Sample", "Interstitial MRAID state : HIDDEN");
						break;
				}
			}
		});

		// instantiate the response handler used when loading an interstitial ad
	    interstitialResponseHandler = new SASAdView.AdResponseHandler() {
			public void adLoadingCompleted(SASAdElement adElement) {
				Log.i("Sample", "Interstitial loading completed");
			}

			public void adLoadingFailed(Exception e) {
				Log.i("Sample", "Interstitial loading failed: " + e.getMessage());
			}
		};
	}
	
	/**
	 * Loads an interstitial ad
	 */
	private void loadInterstitialAd() {
		// load interstitial ad with appropriate parameters (siteID,pageID,formatID,master,targeting,adResponseHandler)
		mInterstitialView.loadAd(35176, "(news_activity)", 12167, true, "startup", interstitialResponseHandler);
	}

	/**
	 * Overriden to clean up SASAdView instances. This must be done to avoid IntentReceiver leak.
	 */
	@Override
	protected void onDestroy() {
		super.onDestroy();
		mBannerView.onDestroy();
		mInterstitialView.onDestroy();
	}
	
	
}
