package com.smartadserver.android.swipenavigationsample;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;

import android.app.ActionBar;
import android.app.FragmentTransaction;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.view.ViewPager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.smartadserver.android.library.SASBannerView;
import com.smartadserver.android.library.ui.SASAdView;
import com.smartadserver.android.library.ui.SASAdView.OnStateChangeListener;
import com.smartadserver.android.library.ui.SASRotatingImageLoader;
import com.smartadserver.android.library.util.SASUtil;

/**
 * Simple activity containing a ViewPager to swipe between several Fragments
 */
public class MainActivity extends FragmentActivity implements ActionBar.TabListener {
	
	// for logging purposes
	private static String TAG = "SWIPE_SAMPLE";

    /**
     * The {@link android.support.v4.view.PagerAdapter} that will provide fragments for each of the
     * three primary sections of the app. We use a {@link android.support.v4.app.FragmentPagerAdapter}
     * derivative, which will keep every loaded fragment in memory. If this becomes too memory
     * intensive, it may be best to switch to a {@link android.support.v4.app.FragmentStatePagerAdapter}.
     */
    AppSectionsPagerAdapter mAppSectionsPagerAdapter;

    /**
     * The {@link ViewPager} that will display the three primary sections of the app, one at a
     * time.
     */
    ViewPager mViewPager;
    

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        
        SASUtil.debugModeEnabled = true;
        SASUtil.enableLogging();

        // Create the adapter that will return a fragment for each of the three primary sections
        // of the app.
        mAppSectionsPagerAdapter = new AppSectionsPagerAdapter(getSupportFragmentManager());

        // Set up the action bar.
        final ActionBar actionBar = getActionBar();
        
        if (actionBar != null) {

        	// Specify that the Home/Up button should not be enabled, since there is no hierarchical
        	// parent.
        	actionBar.setHomeButtonEnabled(false);

        	// Specify that we will be displaying tabs in the action bar.
        	actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);
        }


        // Set up the ViewPager, attaching the adapter and setting up a listener for when the
        // user swipes between sections.
        mViewPager = (ViewPager) findViewById(R.id.pager);
        mViewPager.setAdapter(mAppSectionsPagerAdapter);
        if (actionBar != null) {
        	mViewPager.setOnPageChangeListener(new ViewPager.SimpleOnPageChangeListener() {
        		@Override
        		public void onPageSelected(int position) {
        			// When swiping between different app sections, select the corresponding tab.
        			// We can also use ActionBar.Tab#select() to do this if we have a reference to the
        			// Tab.
        			Log.d(TAG, "onPageSelected:"+position);
        			actionBar.setSelectedNavigationItem(position);
        			
        			// if the selected page is an ad Fragment, request that it start its content.
        			if (mAppSectionsPagerAdapter.isAdPage(position)) {
        				AdSectionFragment adFragment = (AdSectionFragment)mAppSectionsPagerAdapter.getItem(position);
        				adFragment.startContent();
        			}
        			
        			
        		}
        	});

        	// For each of the sections in the app, add a tab to the action bar.
        	for (int i = 0; i < mAppSectionsPagerAdapter.getCount(); i++) {
        		// Create a tab with text corresponding to the page title defined by the adapter.
        		// Also specify this Activity object, which implements the TabListener interface, as the
        		// listener for when this tab is selected.
        		actionBar.addTab(
        				actionBar.newTab()
        				.setText(mAppSectionsPagerAdapter.getPageTitle(i))
        				.setTabListener(this));
        	}
        }
    }


	@Override
    public void onTabUnselected(ActionBar.Tab tab, FragmentTransaction fragmentTransaction) {
    }

    @Override
    public void onTabSelected(ActionBar.Tab tab, FragmentTransaction fragmentTransaction) {
        // When the given tab is selected, switch to the corresponding page in the ViewPager.
        mViewPager.setCurrentItem(tab.getPosition());
    }

    @Override
    public void onTabReselected(ActionBar.Tab tab, FragmentTransaction fragmentTransaction) {
    }
    
    /**
     * Overriden to stop the video when the Activity is paused
     */
    public void onPause() {
		super.onPause();
		int page = mViewPager.getCurrentItem();
		if (mAppSectionsPagerAdapter.isAdPage(page)) {
			AdSectionFragment adFragment = (AdSectionFragment)mAppSectionsPagerAdapter.getItem(page);
			adFragment.stopContent();
		}
	}

    /**
     * Overriden to restart the video when the Activity is resumed on an ad page.
     */
	public void onResume() {
		super.onResume();
		int page = mViewPager.getCurrentItem();
		if (mAppSectionsPagerAdapter.isAdPage(page)) {
			AdSectionFragment adFragment = (AdSectionFragment)mAppSectionsPagerAdapter.getItem(page);
			adFragment.startContent();
		}
	}

    /**
     * A {@link FragmentPagerAdapter} that returns a fragment corresponding to one of the primary
     * sections of the app.
     */
    public static class AppSectionsPagerAdapter extends FragmentPagerAdapter {
    	
    	// declare page count
    	int pageCount = 4;
    	
    	// buffer for created Fragments
    	Fragment[] fragments = new Fragment[pageCount];
    	
    	// declare ad pages indices
    	Integer[] adIndices = {2};
    	HashSet<Integer> adIndicesSet;
    	

        public AppSectionsPagerAdapter(FragmentManager fm) {
            super(fm);
            // init indices for ad views
            adIndicesSet = new HashSet<Integer>((Collection<Integer>)Arrays.asList(adIndices));
        }
        
        public boolean isAdPage(int page) {
        	return adIndicesSet.contains(page);
        }

        @Override
        public Fragment getItem(int i) {
        	Fragment item = null;
        	
        	if (fragments[i] != null) {
        		// return previously created fragment
        		item = fragments[i];
        	} else {
        		// create a new one
        		if (isAdPage(i)) {
            		// this is an Ad view, create an AdSectionFragment
            		item = new AdSectionFragment();
            	} else {
            		// The other sections of the app are dummy content fragments.
            		Fragment fragment = new ContentSectionFragment();
                    Bundle args = new Bundle();
                    args.putInt(ContentSectionFragment.ARG_SECTION_NUMBER, i + 1);
                    fragment.setArguments(args);
                    item = fragment;
            	}
        		
        		// store created fragment
        		fragments[i] = item;
        	}
        	
        	return item;
        }

        @Override
        public int getCount() {
            return pageCount;
        }

        @Override
        public CharSequence getPageTitle(int position) {
        	String title = null;
        	if (isAdPage(position)) {
        		// this is an Ad view
        		title = "Advertising";
        	} else {
        		// The other sections of the app are dummy placeholders.
        		title = "Section " + (position + 1);
        	}
            return title;
        }
    }

    /**
     * A fragment that displays ads.
     */
    public static class AdSectionFragment extends Fragment {

    	public SASBannerView banner;
    	View rootView;
    	
    	boolean startRequired;
    	boolean bannerLoaded;

    	
		/**
		 * Method overriden to be notified when the Fragment becomes visible or gets hidden
		 */
    	@Override
		public void setUserVisibleHint(boolean isVisibleToUser) {
			super.setUserVisibleHint(isVisibleToUser);
			if (!isVisibleToUser) {
    			// when this view is outside the screen, send a message to the adview that it should stop playing any content.
    			// note that this method is available from SAS SDK 4.2 and above. Needs to be commented out otherwise.
				stopContent();
    		}
		}
    	
    	/**
    	 * Send a message to the creative to stop any playing content.
    	 */
    	public synchronized void stopContent() {
    		if (banner != null) {
    			banner.sendMessageToWebView("stop");
    		}
    	}
    	
    	/**
    	 * Send a message to the creative to start any playable content as soon as possible (might need to wait for banner to
    	 * be ready. see OnStateChangeListener below)
    	 */
    	public synchronized void startContent() {
    		if (bannerLoaded) {
    			doStartContent();
    		} else {
    			startRequired = true;
    		}
    	}


        private void doStartContent() {
        	if (banner != null) {
				banner.sendMessageToWebView("start");
			}
		}


		@Override
        public View onCreateView(LayoutInflater inflater, ViewGroup container,
        		Bundle savedInstanceState) {

			rootView = inflater.inflate(R.layout.fragment_section_ad, container, false);
			banner =  (SASBannerView)rootView.findViewById(R.id.banner);
			banner.setLoaderView(new SASRotatingImageLoader(getActivity()));

			// add a state change listener on the banner to know when the creative is ready to react to the 'start' message.
			banner.addStateChangeListener(new OnStateChangeListener() {

				public void onStateChanged(SASAdView.StateChangeEvent stateChangeEvent) {
					synchronized (AdSectionFragment.this) {
						// listen for the VIEW_DEFAULT state change that indicates that the creative is loaded and ready.
						if (stateChangeEvent.getType() == SASAdView.StateChangeEvent.VIEW_DEFAULT) {
							bannerLoaded = true;
							if (startRequired) {
								// if start content was called before, start content now.
								startRequired = false;
								doStartContent();
							}
						}
					}
				}
			});
			
			// reset 2 internal flags, and load ad
			startRequired = false;
			bannerLoaded = false;
			banner.loadAd(28298, "430843", 25090,true,"");
            
            return rootView;
        }

		@Override
		public void onDestroyView() {
			super.onDestroyView();
			
			if (banner != null) {
				banner.onDestroy();
			}
			//((ViewGroup)rootView.getParent()).removeView(rootView);
		}
    }

    /**
     * A dummy fragment representing a content section of the app, but that simply displays dummy text.
     */
    public static class ContentSectionFragment extends Fragment {

        public static final String ARG_SECTION_NUMBER = "section_number";

        @Override
        public View onCreateView(LayoutInflater inflater, ViewGroup container,
                Bundle savedInstanceState) {
            View rootView = inflater.inflate(R.layout.fragment_section_dummy, container, false);
            Bundle args = getArguments();
            ((TextView) rootView.findViewById(android.R.id.text1)).setText(
                    getString(R.string.dummy_section_text, args.getInt(ARG_SECTION_NUMBER)));
            return rootView;
        }
    }
}
