package com.smartadserver.android.vastvideoplayersample;

import java.util.HashMap;

import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnInfoListener;
import android.os.Bundle;
import android.util.Log;
import android.view.Display;
import android.view.KeyEvent;
import android.view.Surface;
import android.view.View;
import android.view.WindowManager;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.RelativeLayout;

import com.smartadserver.android.videolibrary.*;
import com.smartadserver.android.videolibrary.error.SASAdBreakError;
import com.smartadserver.android.videolibrary.error.SASAdError;
import com.smartadserver.android.videolibrary.error.SASVASTAdError;

/**
 * Activity class to play videos with preroll, midroll and postroll ads
 */
public class CompositeVideoPlayerActivity extends Activity implements
		SASVideoView.FullscreenModeHandler {

	private static String TAG = CompositeVideoPlayerActivity.class
			.getSimpleName();

	// VAST ad call config
	public static final int     NETWORK_ID = 92;
    public static final int     SITE_ID = 54094;
    public static final String  PAGE_ID = "391101";
    public static final int     PREROLL_FORMAT_ID = 23539;
    public static final int     MIDROLL_FORMAT_ID = 23540;
    public static final int     MIDROLL_PERCENTAGE_INTERVAL = 50;
    public static final int     POSTROLL_FORMAT_ID = 23541;

	private SASVideoView mVideoView;
	private SASMediaController mMediaController;
	private String mVideoUrl;
	private String mWebsiteUrl;
	private WebView mInfoWebView;

	private SASPlayerAdConfig mPlayerAdConfig;
	private SASPlayerDisplayConfig mPlayerDisplayConfig;

    private MediaPlayer.OnErrorListener mOnErrorListener;
    private SASVideoView.OnAdEventListener mOnAdEventListener;
	private SASVideoView.OnAdOpenedListener mOnAdOpenedListener;
	private SASVideoView.OnAdErrorListener mOnAdErrorListener;

	// variable to store content video position when activity is suspended
	int mCurrentPosition = -1;
	
	// sensor listener to detect orientation changes
	SensorEventListener mSensorEventListener;
	int rotationCountToRelease = -1;

	/**
	 * onCreate initialization method
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_composite_video_player);

		// fetch VideoView from layout file.
		mVideoView = (SASVideoView) this.findViewById(R.id.videoView);
		mInfoWebView = (WebView) this.findViewById(R.id.infoWebView);

		// set a WebViewClient to stay within the webview when clicking on links
		mInfoWebView.setWebViewClient(new WebViewClient() {
			@Override
			public boolean shouldOverrideUrlLoading(WebView view, String url) {
				view.loadUrl(url);
				return false;
			}
		});


		// set mediacontroller on VideoView to display controls
		mMediaController = new SASMediaController(this);
		mVideoView.setMediaController(mMediaController);

		// set this activity as the fullscreen
		mVideoView.setFullscreenModeHandler(this);

		// SensorEventListener to manage Portrait/Landscape orientation changes
		// while Activity orientation is locked.
		mSensorEventListener = new SensorEventListener() {
			int orientation = -1;

			@Override
			public void onSensorChanged(SensorEvent event) {
				Log.i(TAG, "event.values[1]:" + event.values[1]);
				if (rotationCountToRelease > 0) {
					if (event.values[1] < 6.5 && event.values[1] > -6.5) {
						if (orientation == 0) {
							Log.i(TAG, "Sensor Landscape");
							rotationCountToRelease--;
						}
						orientation = 1;
					} else {
						if (orientation == 1) {
							Log.i(TAG, "Sensor Portrait");
							rotationCountToRelease--;
						}
						orientation = 0;
					}

					if (rotationCountToRelease == 0) {
						// 2 changes in a row were detected, now release
						// orientation lock
						setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED);

						mVideoView.post(new Runnable() {
							public void run() {
								SensorManager sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
								sensorManager.unregisterListener(mSensorEventListener);
							}
						});
					}
				}
			}

			@Override
			public void onAccuracyChanged(Sensor sensor, int accuracy) {

			}
		};

		setupLayout();

		// get video url
		Bundle data = getIntent().getExtras();
		mVideoUrl = data.getString(VideoListActivity.INTENT_EXTRA_VIDEO_URL);
		mWebsiteUrl = data.getString(VideoListActivity.INTENT_EXTRA_WEBSITE_URL);

		mInfoWebView.loadUrl(mWebsiteUrl);

		// set various listeners on VideoView
		if (mVideoView != null) {
			mVideoView.setOnPreparedListener(new MediaPlayer.OnPreparedListener() {
				public void onPrepared(MediaPlayer mp) {
				Log.i(TAG, "video onPreparedListenerCalled");
				setupLayout();
				}
			});

			// set completion and info listeners on SASVideoView
			mVideoView.setOnCompletionListener(new OnCompletionListener() {
				public void onCompletion(MediaPlayer mp) {
					Log.i(TAG, "video onCompletionListenerCalled");
					finish();
				}
			});

			mVideoView.setOnInfoListener(new OnInfoListener() {
				public boolean onInfo(MediaPlayer mp, int what, int extra) {
					Log.i(TAG, "video onInfoListenerCalled");
					return false;
				}
			});

			// set ad event listener on SASVideoView for logging purposes
			mOnAdEventListener = new SASVideoView.OnAdEventListener() {

				public void onAdEvent(SASAdEvent event) {
					Log.i(TAG,"video onVideoAdEventListener called. VAST event type:"
									+ event.getEventType()
									+ " VAST sequence type:"
									+ event.getSequenceType() + " duration:"
									+ event.getDuration());
				}
			};
			mVideoView.setOnAdEventListener(mOnAdEventListener);
			
			// set ad error listener on SASVideoView for logging purposes
			mOnAdErrorListener = new SASVideoView.OnAdErrorListener() {
				public void onAdError(SASAdError adError) {
					
					String message = "video onVideoAdErrorListener called.";
					
					// test the actual class of the SASAdError to log extra info, if need be
					if  (adError instanceof SASAdBreakError) {
						Log.i(TAG, "VASTAdBreakError caught : no ad available for " + ((SASAdBreakError)adError).getBreakType());
					} else if  (adError instanceof SASVASTAdError) {
						String breakType = ((SASVASTAdError)adError).getBreakType();
						String insertionId = ((SASVASTAdError)adError).getInsertionId();
						int errorCode = ((SASVASTAdError)adError).getErrorCode();
						Log.i(TAG, "VASTAdError caught during " + breakType + ": VAST error code " + errorCode + " for insertion id " + insertionId);
					}
					
					Log.i(TAG, message);
					
				}
			};
			mVideoView.setOnAdErrorListener(mOnAdErrorListener);

            // set error listener on SASVideoView for logging purposes
            mOnErrorListener = new MediaPlayer.OnErrorListener() {
                @Override
                public boolean onError(MediaPlayer mediaPlayer, int errorCode, int errorExtraCode) {
                    Log.i(TAG, "video onVideoErrorListener called. Error code:" + errorCode + " , extra code:"+errorExtraCode );
                    return false;
                }
            };
            mVideoView.setOnErrorListener(mOnErrorListener);

			// set ad opened listener on SASVideoView for logging purposes
			mOnAdOpenedListener = new SASVideoView.OnAdOpenedListener() {

				public void onAdOpened(String url) {
					Log.i(TAG, "video onVideoAdOpenedListener called for URL: " + url);
				}
			};
			mVideoView.setOnAdOpenedListener(mOnAdOpenedListener);
		}

		// configure VAST ad call parameters
        mPlayerAdConfig = new SASPlayerAdConfig(NETWORK_ID, SITE_ID, PAGE_ID,  PREROLL_FORMAT_ID, MIDROLL_FORMAT_ID, new int[] { MIDROLL_PERCENTAGE_INTERVAL }, POSTROLL_FORMAT_ID, "");
        
        // set extra custom parameters (here syndication params)
        HashMap<String, String> extraParams = new HashMap<String, String>();
        extraParams.put("contentID","myContentID");
        extraParams.put("contentProviderID","myContentProviderID");
        mPlayerAdConfig.setExtraParameters(extraParams);

		// customize ad player 'info' label, for the sake of demonstration
		mPlayerDisplayConfig = new SASPlayerDisplayConfig();
		mPlayerDisplayConfig.infoButtonLabel = "More...";

		mVideoView.setPlayerDisplayConfig(mPlayerDisplayConfig);

		// start content video
		startContentVideo();
	}

	/**
	 * Change Activity layout according to portrait/landscape orientation.
	 */
	private void setupLayout() {
		int orientation = getResources().getConfiguration().orientation;

		if (orientation == Configuration.ORIENTATION_PORTRAIT) {
			mInfoWebView.setVisibility(View.VISIBLE);
			mVideoView.getLayoutParams().height = (int) (240 * getResources().getDisplayMetrics().density);
            mMediaController.setAnchorView(mVideoView);
			mVideoView.setFullscreenState(false);
		} else if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
			mInfoWebView.setVisibility(View.GONE);
			mVideoView.getLayoutParams().height = RelativeLayout.LayoutParams.MATCH_PARENT;
			mVideoView.setFullscreenState(true);
		}
	}

	/**
	 * SASMediaController.FullscreenModeHandler interface implementation. Like
	 * Youtube app, entering fullscreen forces orientation to landscape, but one
	 * subsequent orientation change will revert to free mode. Same for exiting
	 * fullscreen that will force orientation to portrait until next device
	 * rotation.
	 * 
	 */
	public void enterFullscreen() {

		Display display = ((WindowManager) getSystemService(WINDOW_SERVICE))
				.getDefaultDisplay();
		int orientation = display.getRotation();

		if (orientation == Surface.ROTATION_270) {
			setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE);
		} else {
			setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		}
		setOrientationReleaseTrigger();
	}

	public void exitFullscreen() {
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		setOrientationReleaseTrigger();
	}

	/**
	 * Set internal orientation release trigger
	 */
	private void setOrientationReleaseTrigger() {
		SensorManager sensorManager = (SensorManager)this.getSystemService(Context.SENSOR_SERVICE);
		sensorManager.registerListener(mSensorEventListener,
				sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),
				SensorManager.SENSOR_DELAY_NORMAL);
		rotationCountToRelease = 2;
	}

	/**
	 * Change application layout according to portrait/landscape device
	 * orientation
	 */
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);

		setupLayout();
	}

	/**
	 * Configure the VideoView of this activity and starts playing content.
	 */
	private void startContentVideo() {
		// this method might me executed from the interstitial loading thread,
		// so make sure
		// the following code gets executed from UI thread
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				mVideoView.setVideoPath(mVideoUrl, mPlayerAdConfig);
				mVideoView.start();
			}
		});
	}
	

	/**
	 * Force finish of this Activity on back key pressed
	 */
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			this.finish();
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}

	/**
	 * Overridden onDestroy method to clean up SASInterstitialView and free resources
	 */
	@Override
	protected void onDestroy() {
		super.onDestroy();
		mVideoView.onDestroy();
	}

	/**
	 * Overridden onPause() method to perform specific tasks when the {@link CompositeVideoPlayerActivity} is suspended. 
	 */
	@Override
	protected void onPause() {
		super.onPause();
		// notify the SASVideoView that it should suspend video ad playback.
		mVideoView.onPause();
		// memorize the video position to restore it when the Activity resumes.
		mCurrentPosition = mVideoView.getCurrentPosition();
		SensorManager sensorManager = (SensorManager) this.getSystemService(Context.SENSOR_SERVICE);
		sensorManager.unregisterListener(mSensorEventListener);
	}

	/**
	 * Overridden onResume() method to perform specific tasks when the {@link CompositeVideoPlayerActivity} resumes. 
	 */
	@Override
	protected void onResume() {
		super.onResume();
		// notify the SASVideoView that it should resumes video ad playback.
		mVideoView.onResume();
		// restore the video position where it was suspended if possible.
		if (mCurrentPosition > -1 && mVideoView.canSeekForward()) {
			mVideoView.seekTo(mCurrentPosition);
		}
		// set orientation release trigger if need be
		if (rotationCountToRelease > 0) {
			setOrientationReleaseTrigger();
		}
	}
}
