package com.smartadserver.android.vastvideoplayersample;


import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.ListActivity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

/**
 * Activity displaying a list of videos
 */
public class VideoListActivity extends ListActivity {
	
	// constant property keys
	public static final String INTENT_EXTRA_VIDEO_URL = "videoUrl";
	public static final String INTENT_EXTRA_WEBSITE_URL = "websiteUrl";
	
	// LayoutInflater to instantiate ListView elements
	private LayoutInflater mInflater;
	
	/**
	 * class holding video element description
	 */
	private static class VideoElement {
		// video title
		public String title;
		// video details
		public String details;
		// video file Url
		public String videoUrl;
		// video poster Url
		public String posterUrl;
		// video website Url
		public String websiteUrl;
		// video element poster drawable
		public Drawable posterDrawable;
		
		public VideoElement(String title, String details, String videoUrl, String posterUrl, String websiteUrl) {
			this.title = title;
			this.details = details;
			this.videoUrl = videoUrl;
			this.posterUrl = posterUrl;
			this.websiteUrl = websiteUrl;
		}
	}
	
	/**
	 * Custom adapter to display video items with poster icon in ListView
	 */
	private class CustomAdapter extends ArrayAdapter<VideoElement> {

		/**
		 * Constructor
		 */
		public CustomAdapter(Context context, int resource,
				int textViewResourceId, List<VideoElement> objects) {               

			super(context, resource, textViewResourceId, objects);
		}
		
		
		/**
		 * Overriden method to update view with list item parameters
		 */
		@Override
		public View getView(int position, View convertView, ViewGroup parent) {   

			final ViewHolder holder;
			final VideoElement videoElement= getItem(position);
			if(convertView == null){
				convertView = mInflater.inflate(R.layout.video_list_item, null);
				holder = new ViewHolder();
				holder.title = (TextView) convertView.findViewById(R.id.title);
				holder.detail = (TextView) convertView.findViewById(R.id.details);
				holder.image = (ImageView) convertView.findViewById(R.id.img);
				convertView.setTag(holder);
			} else {
				holder = (ViewHolder)convertView.getTag();
			}
			
			holder.title.setText(videoElement.title);
			holder.detail.setText(videoElement.details);
			
			// width and height of image icon
			final int vWidth = holder.image.getLayoutParams().width;
			final int vHeight = holder.image.getLayoutParams().height;
			
			
			// need to fetch poster for this item if null
			if (videoElement.posterDrawable == null) {
				// set temporary transparent drawable to mark this item as being downloaded
				videoElement.posterDrawable = getResources().getDrawable(android.R.color.transparent);
				
				// create download thread
				Thread imageLoader = new Thread() {
					public void run() {
						// fetch image drawable on this thread
						final Drawable drawable = imageFromUrl(videoElement.posterUrl);

						if (drawable != null) {
							// scale this image to match list item icon size
							Bitmap bitmap = ((BitmapDrawable)drawable).getBitmap();
							int bWidth = bitmap.getWidth();
							int bHeight = bitmap.getHeight();
							double resizeRatio = Math.min(vWidth / (double)bWidth, vHeight / (double)bHeight);
							Bitmap scaledBitmap = Bitmap.createScaledBitmap(bitmap, (int)(bWidth * resizeRatio), (int)(bHeight * resizeRatio), true);

							// now store scaled bitmap in video element 
							videoElement.posterDrawable = new BitmapDrawable(getResources(), scaledBitmap);

							// ...then refresh image view in list
							runOnUiThread(new Runnable() {
								public void run() {
									notifyDataSetChanged();
								}
							});
						}
					}
				};
				// start image download thread
				imageLoader.start();			
			} else {
				// the drawable is available, set it on the list item icon
				holder.image.setImageDrawable(videoElement.posterDrawable);
			}
			
			return convertView;
		}
		
		/**
		 * holder pattern class to cache View's children for fast access (rather that findViewById)
		 */
		private class ViewHolder {
			public TextView title = null;
			public TextView detail = null;
			public ImageView image=null;
		}
	}

	/**
	 * onCreate initialization method
	 */
	@SuppressLint("NewApi")
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

        // Activate remote debugging with Chrome on Android KitKat (only if built in DEBUG mode)
        /*if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            if ( 0 != ( getApplicationInfo().flags &= ApplicationInfo.FLAG_DEBUGGABLE ) ) {
                WebView.setWebContentsDebuggingEnabled(true);
            }
        }   */

        setContentView(R.layout.activity_video_list);
		
		// initialize inflater instance
		mInflater = (LayoutInflater) getSystemService(Activity.LAYOUT_INFLATER_SERVICE);

		// create CustomAdapter and set it on the ListView
		List<VideoElement> videoList = getVideolist();
		final CustomAdapter adapter = new CustomAdapter(this, R.layout.video_list_item,R.id.title, videoList);
		setListAdapter(adapter);

	}
	
	/**
	 * Overriden method to launch VideoPlayerActivity when a video item is clicked.
	 */
	public void onListItemClick(ListView parent, View v, int position,
			long id) {        	

		String videoUrl = ((VideoElement)parent.getItemAtPosition(position)).videoUrl;
		String websiteUrl = ((VideoElement)parent.getItemAtPosition(position)).websiteUrl;
		
		// create VideoPlayerActivity intent, sets the extra parameters (url) on it.
		Intent playVideoIntent = new Intent(this, CompositeVideoPlayerActivity.class);
        Bundle parameters = new Bundle();
        parameters.putString(INTENT_EXTRA_VIDEO_URL, videoUrl);
        parameters.putString(INTENT_EXTRA_WEBSITE_URL, websiteUrl);
        playVideoIntent.putExtras(parameters);
        
        // start activity
        startActivity(playVideoIntent);
		
	}

	/**
	 * Returns a list of VideoElement instances (data source for the ListView)
	 */
	private List<VideoElement> getVideolist() {
		ArrayList<VideoElement> list = new ArrayList<VideoElement>();
		list.add(new VideoElement("Big Buck Bunny trailer",
				"Directed by: Sacha Goedegebure\nRelease year: 2008" ,
				"http://ak-ns.sascdn.com/mobilesdk/samples/videos/BigBuckBunnyTrailer_480x320.mp4",
				"http://ak-ns.sascdn.com/mobilesdk/samples/videos/BigBuckBunny.jpg",
				"http://m.imdb.com/title/tt1254207/"));
		list.add(new VideoElement("Sintel trailer",
				"Directed by: Colin Levy\nRelease year: 2010" ,
				"http://ak-ns.sascdn.com/mobilesdk/samples/videos/SintelTrailer_480x320.mp4",
				"http://ak-ns.sascdn.com/mobilesdk/samples/videos/Sintel.jpg",
				"http://m.imdb.com/title/tt1727587/"));
		list.add(new VideoElement("Elephants Dream",
				"Directed by: Bassam Kurdali\nRelease year: 2006" ,
				"http://ak-ns.sascdn.com/mobilesdk/samples/videos/ElephantsDreamTeaser_568x320.mp4",
				"http://ak-ns.sascdn.com/mobilesdk/samples/videos/ElephantsDream.jpg",
				"http://m.imdb.com/title/tt0807840/"));
		return list;
	}
	
	/**
	 * Convenience method to create a Drawable from a url.
	 */
	private static Drawable imageFromUrl(String url)
    {
        try
        {
            InputStream is = (InputStream) new URL(url).getContent();
            Drawable d = Drawable.createFromStream(is, "src name");
            return d;
        }catch (Exception e) {
            System.out.println("Exc="+e);
            return null;
        }
    }
}
