//
//  BannerInListViewController.swift
//  SwiftSample
//
//  Created by Loïc GIRON DIT METAZ on 05/05/2015.
//  Copyright (c) 2015 Smart AdServer. All rights reserved.
//

import AVFoundation


class BannerInTableViewController: UITableViewController, SASAdViewDelegate {
    
    let banner1Row = 5
    let banner2Row = 15
    let numberOfCells = 30
    
    let defaultCellHeight: CGFloat = 50.0
    
    // Instances of banners (marked as optional since it is created after the initialization of the controller)
    var banner1: SASBannerView?
    var banner2: SASBannerView?
    
    var statusBarHidden = false
    
    // MARK: - View controller lifecycle
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        createReloadButton()
        createBanners()
        
        // Registering cell identifier for content (ad cells identifiers are automatically handled by the ad view container cell and should not be declared)
        tableView.registerClass(UITableViewCell.self, forCellReuseIdentifier: "ContentCell")
    }
    
    override func prefersStatusBarHidden() -> Bool {
        // The status bar can be hidden by an ad (for instance when the ad is expanded in fullscreen)
        return statusBarHidden || super.prefersStatusBarHidden()
    }
    
    func createReloadButton() {
        navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Reload", style: .Bordered, target: self, action: #selector(BannerInTableViewController.reload))
    }
    
    // MARK: - Ad management
    
    func reload() {
        self.removeBanners()
        self.createBanners()
    }
    
    func createBanners() {
        self.banner1 = createBanner(Constants.banner1InTableViewPageID())
        self.banner2 = createBanner(Constants.banner2InTableViewPageID())
    }
    
    func createBanner(pageId: String) -> SASBannerView {
        // The instance of the banner is created with a default frame and an appropriate loader.
        let banner = SASBannerView(frame: bannerFrameForView(self.view, height: defaultCellHeight), loader: .ActivityIndicatorStyleWhite)
        
        // Setting the delegate.
        banner.delegate = self
        
        // Setting the modal parent view controller.
        banner.modalParentViewController = self
        
        // Loading the ad (using IDs from the Constants class).
        banner.loadFormatId(Constants.bannerFormatID(), pageId: pageId, master: true, target: nil)
        
        return banner
    }
    
    func removeBanners() {
        removeBanner(banner1)
        removeBanner(banner2)
    }
    
    func removeBanner(banner: SASBannerView?) {
        banner?.removeFromSuperview()
    }
    
    func bannerFrameForView(view: UIView, height: CGFloat) -> CGRect {
        // Convenient method to reset the frame of a banner
        return CGRect(x: 0, y: 0, width: CGRectGetWidth(view.frame), height: height)
    }
    
    // MARK: - Table View data source & delegate
    
    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        var height: CGFloat = defaultCellHeight
        
        // If the cell is an ad cell, the actual cell height can be computed easily using the optimalAdViewHeightForContainer: method
        if indexPath.row == banner1Row, let banner1Height = banner1?.optimalAdViewHeightForContainer(tableView) {
            height = banner1Height
        } else if indexPath.row == banner2Row, let banner2Height = banner2?.optimalAdViewHeightForContainer(tableView) {
            height = banner2Height
        }
        
        return height
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return numberOfCells
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        if isAdCell(indexPath) {
            // Attach relevant banner to the container cell
            switch (indexPath.row) {
            case banner1Row:
                return SASAdViewContainerCell(forAdView: banner1, inTableView: tableView)
            case banner2Row:
                return SASAdViewContainerCell(forAdView: banner2, inTableView: tableView)
            default:
                NSLog("Error: cell should not be an AdCell")
                return tableView.dequeueReusableCellWithIdentifier("ContentCell")!
            }
        } else {
            // Configure regular cells
            let cell = tableView.dequeueReusableCellWithIdentifier("ContentCell")!
            cell.textLabel?.text = "Lorem ipsum dolor sit amet"
            cell.accessoryType = UITableViewCellAccessoryType.DisclosureIndicator
            return cell
        }
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        //Deselect the cell if it's an ad to avoid messing with the ad creative.
        if (indexPath.row == banner1Row || banner2Row == indexPath.row) {
            //Ad Cell was selected, nothing to do here.
        } else {
            //Your normal behavior on cell click like pushing a new VC
            pushViewController()
        }
        tableView.deselectRowAtIndexPath(indexPath, animated: false)
    }
    
    func pushViewController() {
        let controller = UIViewController()
        controller.navigationItem.title = "Details"
        controller.view.backgroundColor = UIColor.whiteColor()
        let label = UILabel(frame: controller.view.bounds)
        label.autoresizingMask = [UIViewAutoresizing.FlexibleWidth, UIViewAutoresizing.FlexibleHeight]
        label.textAlignment = NSTextAlignment.Center
        label.text = "Detail view controller"
        controller.view.addSubview(label)
        navigationController?.pushViewController(controller, animated: true)
    }
    
    func isAdCell(indexPath: NSIndexPath) -> Bool {
        // In this sample, ad cells are in fixed positions
        return (indexPath.row == banner1Row || indexPath.row == banner2Row)
    }
    
    func refreshAdCell(adView: SASAdView?) {
        // Refresh the ad cell (or the whole table view if no ad cell is given in parameter)
        if adView == banner1 {
            tableView.reloadRowsAtIndexPaths([NSIndexPath(forRow: banner1Row, inSection: 0)], withRowAnimation: .Automatic)
        } else if adView == banner2 {
            tableView.reloadRowsAtIndexPaths([NSIndexPath(forRow: banner2Row, inSection: 0)], withRowAnimation: .Automatic)
        } else {
            tableView.reloadData()
        }
    }
    
    // MARK: - SASAdViewDelegate
    
    func adView(adView: SASAdView, didDownloadAd ad: SASAd) {
        NSLog("An ad object has been loaded")
        
        // Refresh the table view when an ad is loaded
        refreshAdCell(adView)
    }
    
    func adViewDidLoad(adView: SASAdView)  {
        NSLog("Banner has been loaded")
    }
    
    func adView(adView: SASAdView, didFailToLoadWithError error: NSError) {
        NSLog("Banner has failed to load with error: \(error.description)")
        
        // The cell is collapsed if the ad cannot be loaded
        removeBanner(adView as? SASBannerView)
        refreshAdCell(adView)
    }
    
    func adViewWillExpand(adView: SASAdView) {
        NSLog("Banner will expand")
        
        // When the ad is expanded, hide the status bar
        statusBarHidden = true;
        setNeedsStatusBarAppearanceUpdate()
    }
    
    func adViewWillCloseExpand(adView: SASAdView) {
        NSLog("Banner will close expand")
        
        // When the ad will closed, the status bar should be restored
        statusBarHidden = false;
        setNeedsStatusBarAppearanceUpdate()
    }
    
}
