//
//  DetailViewController.m
//  VASTVideoSample
//
//  Created by Clémence Laurent on 29/04/13.
//  Copyright (c) 2013 Smart AdServer. All rights reserved.
//

#import "FullscreenOnlyVideoViewController.h"
#import "SASPlayerAdConfig.h"
#import "SASPlayerMidrollIntervalConfig.h"
#import "SASPlayerMidrollPercentageConfig.h"
#import "SASPlayerDisplayConfig.h"
#import "SASPlayerErrors.h"
#import "SASAdError.h"
#import "SASAdPlayerError.h"
#import "SASVASTAdError.h"
#import "SASAdBreakError.h"
#import "Constants.h"
#import "AppDelegate.h"

#define kErrorAlertViewTag	1


@implementation FullscreenOnlyVideoViewController

#pragma mark - Deallocation

- (void)dealloc {	
	[self removeObservers];
	[self releaseInstances];
		
	[super dealloc];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
	
	if (self.isViewLoaded && self.view.window == nil) {
		self.view = nil;
	}
	if (!self.isViewLoaded) {
		[self releaseInstances];
	}
}


- (void)releaseInstances {
	
	// The delegate needs to be set to nil before before releasing the view controller to avoid crashes.
	_moviePlayerViewController.delegate = nil;
	[_moviePlayerViewController release], _moviePlayerViewController = nil;
	
	[_videoURL release], _videoURL = nil;
}

#pragma mark - View life cycle

- (id)initWithVideoURL:(NSURL *)videoURL {
	self = [super init];
    if (self) {
		self.wantsFullScreenLayout = YES;
		_videoURL = [videoURL retain];
	}
    return self;
}


- (void)viewDidLoad {
    [super viewDidLoad];
	
    if ([self respondsToSelector:@selector(setNeedsStatusBarAppearanceUpdate)]) {
		[self setNeedsStatusBarAppearanceUpdate];
	}
	[self addObservers];
	[self displayMovieViewController];
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
	
	// The status bar is hidden at start to provide a fullscreen experience.
	[[UIApplication sharedApplication] setStatusBarHidden:YES withAnimation:UIStatusBarAnimationFade];
}


- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:YES];
	
	// Remove timeout timer if still existing
	[NSObject cancelPreviousPerformRequestsWithTarget:self];
	
	// The status bar is restored before leaving the view controller.
	[[UIApplication sharedApplication] setStatusBarHidden:NO withAnimation:UIStatusBarAnimationFade];
}

#pragma mark - Movie player

- (void)displayMovieViewController {
	
	// Do not load video player if no URL is available
	if (!_videoURL) {
		return;
	}
	
	// Video player initialization and configuration.
	// The player rely on two separate config objects: SASPlayerAdConfig & SASPlayerDisplayConfig that are used for initialization.
	if (!_moviePlayerViewController) {
		
        // SASPlayerAdConfig
		// --------------------------------------------------------------------------------------------------------------------------
		
		// The ad configuration is used to set IDs that will be used to fetch the ad on Smart AdServer. You have to provide a networkID, a siteID,
		// a pageID, a formatID for each type of ad break (preroll/midroll/postroll).
		// Without a valid ad configuration, the video player will not query Smart AdServer and act as a regular video player.
		
		// It is possible to set each formatID independently using the dedicated setter methods. Specifying a target and a timeout
		// is optional, all other parameters are required.
        SASPlayerAdConfig *playerAdConfig = [SASPlayerAdConfig adConfigWithNetworkID:kNetworkID siteID:kSiteID pageID:kPageID target:kTarget timeout:kTimeout];
        
        [playerAdConfig setPrerollWithFormatID:kPrerollFormatID];
        [playerAdConfig setPostrollWithFormatID:kPostrollFormatID];
		
		// Setting a midroll requires a midroll configuration that will define where the midroll is supposed to be played.
		
		// The midroll configuration can be defined using an array of percentages: this percentages represents place in the video where
		// midrolls will be played (if available).
        [playerAdConfig setMidrollWithFormatID:kMidrollFormatID midrollConfig:[SASPlayerMidrollPercentageConfig midrollConfigWithPercentages:[NSArray arrayWithObject:kMidrollConfigMiddleInterval]]];
		
		// The midroll configuration can be defined using a time interval (in seconds): midrolls will play each time this interval has been reached.
		//[playerAdConfig setMidrollWithFormatID:kMidrollFormatID midrollConfig:[SASPlayerMidrollIntervalConfig midrollConfigWithInterval:10.]];
		
		// It is possible to set the same formatID for all ad break type using this method:
		// SASPlayerAdConfig *playerAdConfig = [SASPlayerAdConfig adConfigWithNetworkID:kNetworkID siteID:kSiteID pageID:kPageID formatID:kPrerollFormatID midrollConfig:[SASPlayerMidrollPercentageConfig midrollConfigWithPercentages:[NSArray arrayWithObject:kMidrollConfigMiddleInterval]] target:kTarget];
		
        
        // SASPlayerDisplayConfig
		// --------------------------------------------------------------------------------------------------------------------------
		
		// The player configuration is used to set the messages that will be displayed by the ad player. If no valid player configuration
		// is provided during video player initialization, default messages will be used.
		// Countdown message and Skip message can contain a countdown. You choose the countdown place in your string by adding '{0}'.
		
		// It is possible to assign different countdown messages depending of the ad break type using dedicated setter methods.
		// When you create a player configuration object, every message set to nil will be replaced by the player default message.
        SASPlayerDisplayConfig *playerDisplayConfig = [SASPlayerDisplayConfig displayConfigWithAdCountdownMessage:nil skipMessage:nil skipButtonLabel:@"Skip" infoButtonLabel:@"Info >>" doneButtonLabel:@"Close"];
		[playerDisplayConfig setPrerollAdCountdownMessage:@"Your video will start in {0}"];
        [playerDisplayConfig setMidrollAdCountdownMessage:@"Your video will resume in {0}"];
        [playerDisplayConfig setPostrollAdCountdownMessage:@"You will get back to your app in {0}"];
		
		// It is possible to assign an unique countdown message using this method:
		//SASPlayerDisplayConfig *playerDisplayConfig = [SASPlayerDisplayConfig displayConfigWithAdCountdownMessage:nil skipMessage:@"You can skip this ad in {0}s" skipButtonLabel:@"Skip" infoButtonLabel:@"Info >>" doneButtonLabel:@"Close"];
		
		
		// SASMoviePlayerViewController
		// --------------------------------------------------------------------------------------------------------------------------
		
		// The video player is initialized using the three configuration objects and an URL to the content video.
		// - the content URL is mandatory.
		// - the ad configuration is mandatory if you want to display ads: the video player will only play the content video if you
		//   set it to nil or if you pass an invalid object.
		// - the player configuration is optional: if set to nil, the ad player will use its default configuration.
		// - the display mode: in this case we set it to SASMoviePlayerControlsDisplayModeFullscreenOnly (see DefaultAndInlineViewController
		//   for additional options).
		
		SASMoviePlayerViewController *moviePlayerViewController = [[SASMoviePlayerViewController alloc] initWithContentURL:_videoURL adConfig:playerAdConfig displayConfig:playerDisplayConfig displayMode:SASMoviePlayerControlsDisplayModeFullscreenOnly];
		_moviePlayerViewController = [moviePlayerViewController retain];
		
		// The modal parent view controller property is used to display the post click modal view, if the user clicks on the Info button.
		// If no modal parent view controller is defined, the link will be opened in Safari instead.
		_moviePlayerViewController.modalParentViewController = self;
		
		// The delegate allows the application to catch several events (see below).
		// Remember to set this delegate to nil before deallocating the view controller.
		_moviePlayerViewController.delegate = self;
		
		// Controls on CONTENT video player are activated (and can be deactivated using NO). The default value is YES.
		// Controls on ad video player cannot be deactivated!
		// Be careful when disabling controls in Fullscreen Only display mode that you still provide an option to exit the screen.
		_moviePlayerViewController.moviePlayer.controlsEnabled = YES;
		
		// It is possible to deactivate the 'Done' button for a particular ad break type.
		_moviePlayerViewController.moviePlayer.prerollDoneDisabled = NO;
		_moviePlayerViewController.moviePlayer.midrollDoneDisabled = YES;
		_moviePlayerViewController.moviePlayer.postrollDoneDisabled = NO;
		
		[moviePlayerViewController release];
	}
	
	// Frame and autoresizing configuration.
	_moviePlayerViewController.moviePlayer.view.frame = self.view.frame;
	_moviePlayerViewController.moviePlayer.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	
	// Prepare and launch video.
	[_moviePlayerViewController.moviePlayer prepareToPlay];
	
	// Add timeout for the content video (please note that the integrated timeout mechanism is only working on advertisements, not on the content video).
	_contentVideoLoaded = NO;
	[self performSelector:@selector(contentVideoTimeout) withObject:nil afterDelay:kContentVideoTimeout];

	// Add the player to the view.
	[self.view addSubview:_moviePlayerViewController.moviePlayer.view];
}

#pragma mark - Movie player notifications

- (void)addObservers {
	
	// Standard MPMoviePlayer notifications
	// --------------------------------------------------------------------------------------------------------------------------
	// Since the moviePlayer is a subclass of MPMoviePlayerController, you can add observers on it for any MPMoviePlayer
	// standard notifications.
	// However, since the video SDK will automatically call play/pause/stop on the MPMoviePlayerController for you during ad
	// breaks, the notifications could be sent at unexpected times.
	
	// Standard MPMoviePlayer notification to handle changes in load state (video playable/video stalled).
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(movieLoadStateDidChange:)
												 name:MPMoviePlayerLoadStateDidChangeNotification
											   object:_moviePlayerViewController];
	
	// Standard MPMoviePlayer notification to handle playback did finish (video playback error/stopped).
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(moviePlaybackDidFinish:)
                                                 name:MPMoviePlayerPlaybackDidFinishNotification
                                               object:_moviePlayerViewController.moviePlayer];
	
	
	// SASMoviePlayerController notifications
	// --------------------------------------------------------------------------------------------------------------------------
	// The video SDK defines severals notifications that will be useful to listen for the behavior of the player or for the
	// current state of an ad playing.
	
	// Notification sent when an ad break starts. The userInfo contains a parameter indicating the type of ad break.
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(movieAdBreakStart:)
												 name:SASMoviePlayerControllerAdBreakStartNotification
											   object:_moviePlayerViewController.moviePlayer];
	
	// Notification sent when an ad break ends. The userInfo contains a parameter indicating the type of ad break.
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(movieAdBreakEnd:)
												 name:SASMoviePlayerControllerAdBreakEndNotification
											   object:_moviePlayerViewController.moviePlayer];
	
	// Notification sent when the whole video content has been played (content video + ads).
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(movieAdVideoSessionDidComplete:)
												 name:SASMoviePlayerControllerVideoSessionDidCompleteNotification
											   object:_moviePlayerViewController.moviePlayer];
}


- (void)removeObservers {
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:MPMoviePlayerLoadStateDidChangeNotification
												  object:_moviePlayerViewController];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:MPMoviePlayerPlaybackDidFinishNotification
												  object:_moviePlayerViewController];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:SASMoviePlayerControllerAdBreakStartNotification
												  object:_moviePlayerViewController];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:SASMoviePlayerControllerAdBreakEndNotification
												  object:_moviePlayerViewController];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:SASMoviePlayerControllerVideoSessionDidCompleteNotification
												  object:_moviePlayerViewController];
}

#pragma mark - Movie player notification

- (void)movieLoadStateDidChange:(NSNotification *)aNotification {
	
	// Play the video when ready, pause the video if there is buffering issues.
	MPMoviePlayerController *player = ((MPMoviePlayerController *)aNotification.object);
	if (player.loadState == MPMovieLoadStateUnknown || player.loadState == MPMovieLoadStateStalled) {
		[player pause]; 
	} else if (player.loadState == MPMovieLoadStatePlayable || player.loadState == MPMovieLoadStatePlaythroughOK) {
		_contentVideoLoaded = YES;
        [player play];
    }
}


- (void)moviePlaybackDidFinish:(NSNotification *)aNotification {
	
    // We need to determine if playback did finish due to an error or if it was stopped to deliver ad break
    NSDictionary *notificationUserInfo = [aNotification userInfo];
    NSNumber *resultValue = [notificationUserInfo objectForKey:MPMoviePlayerPlaybackDidFinishReasonUserInfoKey];
    MPMovieFinishReason reason = [resultValue intValue];
	
    if (reason == MPMovieFinishReasonPlaybackError) {
		
		// If there is a playback error, the actual cause of the error can be retrieved from the notification.
		NSError *mediaPlayerError = [notificationUserInfo objectForKey:@"error"];
        NSString *errorMessage = nil;
        if (mediaPlayerError) {
            errorMessage = [mediaPlayerError localizedDescription];
        } else {
            errorMessage = @"Playback failed without any given reason";
        }
        
		NSLog(@"moviePlaybackDidFinish with error: %@", errorMessage);
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle: @"Playback failed" message:errorMessage delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil];
		alert.tag = kErrorAlertViewTag;
		alert.delegate = self;
        [alert show];
        [alert release];
		
    } else {
		
		// When an ad break starts, the video player is stopped to allow the video ad to be played (only one video
		// can be played at the same time).
		// Always remember to check the actual reason if you listen for the 'did finish' notification.
        NSLog(@"moviePlaybackDidFinish: player stop due to ad break");
		
		// If you simply want to do something when your video is finished (including advertisement), you should use
		// SASMoviePlayerControllerVideoSessionDidCompleteNotification instead.
    }
}


- (void)movieAdBreakStart:(NSNotification *)notification {
	// It is possible to extract the ad break type from the notification with the key SASMoviePlayerControllerAdBreakNotificationType
	NSString *adBreakType = [[notification userInfo] objectForKey:SASMoviePlayerControllerAdBreakNotificationType];
	
	// The ad break type is represented by a NSString.
	// If you want to test what type of ad break is encountered, use the macro representation of the ad break type
	// instead of using a string to prevent issue with future SDK releases. Valid macros are:
	// - SASMoviePlayerControllerAdBreakNotificationTypePreroll
	// - SASMoviePlayerControllerAdBreakNotificationTypeMidroll
	// - SASMoviePlayerControllerAdBreakNotificationTypePostroll
	NSLog(@"movieAdBreakStart: %@", adBreakType);
}


- (void)movieAdBreakEnd:(NSNotification *)notification {
	// It is possible to extract the ad break type from the notification with the key SASMoviePlayerControllerAdBreakNotificationType
	NSString *adBreakType = [[notification userInfo] objectForKey:SASMoviePlayerControllerAdBreakNotificationType];
	
	// The ad break type is represented by a NSString.
	// If you want to test what type of ad break is encountered, use the macro representation of the ad break type
	// instead of using a string to prevent issue with future SDK releases. Valid macros are:
	// - SASMoviePlayerControllerAdBreakNotificationTypePreroll
	// - SASMoviePlayerControllerAdBreakNotificationTypeMidroll
	// - SASMoviePlayerControllerAdBreakNotificationTypePostroll
	NSLog(@"movieAdBreakEnd: %@", adBreakType);
}


- (void)movieAdVideoSessionDidComplete:(NSNotification *)notification {
	NSLog(@"movieAdVideoSessionDidComplete:");
	
	// When the video and the postroll (if any) are played, the player modal view controller can be dismissed
	[self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Movie player view controller delegate

- (void)moviePlayerViewController:(SASMoviePlayerViewController *)moviePlayerViewController didFailToLoadAd:(NSError *)error {
	// This delegate is called when an ad failed to load.
	// The error indicates the cause of the failure.
	NSLog(@"moviePlayerViewController:didFailToLoadAd: %@", error);
	
	//If you encounter a VAST specific error, you can retrieve the VAST error code and the ID of the responsible insertion (if available).
	// The complete list of VAST error code is available in the official VAST 3.0 documentation:
	// http://www.iab.net/media/file/VASTv3.0.pdf
	//
	// Please note that receiving a VAST error does not mean that no ad has been displayed.
	if ([error.domain isEqualToString:kSASVASTAdErrorDomain]) {
		NSString *breakType = [[error userInfo] objectForKey:kSASAdErrorBreakTypeKey];
		NSNumber *VASTErrorCode = [NSNumber numberWithInteger:error.code];
		NSNumber *insertionID = [[error userInfo] objectForKey:kSASAdErrorInsertionIDKey];
		NSLog(@"VAST Error - breakType: %@ / error code: %ld / insertion id: %ld", breakType, (long)[VASTErrorCode integerValue], (long)[insertionID integerValue]);
	}
	
	// If the error is of type AdBreakError, a whole ad break has failed.
	// When an ad break is cancelled due to errors, the content video is automatically resumed (except for postroll).
	else if ([error.domain isEqualToString:kSASAdBreakErrorDomain]) {
		NSString *breakType = [[error userInfo] objectForKey:kSASAdErrorBreakTypeKey];
		NSLog(@"Ad Break Error - breakType: %@", breakType);
	}
}


- (void)moviePlayerViewController:(SASMoviePlayerViewController *)moviePlayerViewController willPerformActionWithExit:(BOOL)exit {
	// This delegate is called when the ad is clicked.
	// The boolean indicates if the application will be exited to perform the click action (redirection to Safari, Youtube, Plan, …).
	NSLog(@"moviePlayerViewController:willPerformActionWithExit: %@", exit ? @"YES" : @"NO");
}


- (void)moviePlayerViewControllerWillPresentModalView:(SASMoviePlayerViewController *)moviePlayerViewController {
	// This delegate is called when the video player will present a modal view over your content.
	// The modal view will only be displayed if the modalParentViewController property is defined.
	NSLog(@"moviePlayerViewControllerWillPresentModalView:");
}


- (void)moviePlayerViewControllerWillDismissModalView:(SASMoviePlayerViewController *)moviePlayerViewController {
	// This delegate is called when the modal view over your content is dismissed.
	NSLog(@"moviePlayerViewControllerWillDismissModalView:");
}

#pragma mark - Movie player timeout

- (void)contentVideoTimeout {
	
	// Exit the controller if contentVideo cannot be loaded
	if (!_contentVideoLoaded) {
		
		UIAlertView *alert = [[UIAlertView alloc] initWithTitle: @"Playback Failed" message:@"Timeout when loading the content video" delegate:nil cancelButtonTitle:@"Ok" otherButtonTitles:nil];
        [alert show];
        [alert release];
		
		[self dismissViewControllerAnimated:YES completion:nil];
	}
}

#pragma mark - Alert view delegate

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	
	if (alertView.tag == kErrorAlertViewTag) {
		// In case of error on the content video, the whole view controller should be dismissed.
		[self dismissViewControllerAnimated:YES completion:nil];
	}
}

@end
