//
//  SASMoviePlayerViewController.h
//  SASVideoSDK
//
//  Created by Loïc GIRON DIT METAZ on 09/08/13.
//  Copyright (c) 2013 Smart AdServer. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "SASMoviePlayerController.h"

// SASMoviePlayerController public notifications
#define SASMoviePlayerControllerAdBreakStartNotification			@"SASMoviePlayerControllerAdBreakStartNotification"
#define SASMoviePlayerControllerAdBreakEndNotification				@"SASMoviePlayerControllerAdBreakEndNotification"
#define SASMoviePlayerControllerDidEnterFullscreenNotification		@"SASMoviePlayerControllerDidEnterFullscreenNotification"
#define SASMoviePlayerControllerDidExitFullscreenNotification		@"SASMoviePlayerControllerDidExitFullscreenNotification"
#define SASMoviePlayerControllerVideoSessionDidCompleteNotification	@"SASMoviePlayerControllerVideoSessionDidCompleteNotification"

// Ad break types (for SASMoviePlayerControllerAdBreakStartNotification & SASMoviePlayerControllerAdBreakEndNotification notifications).
#define SASMoviePlayerControllerAdBreakNotificationType				@"state"
#define SASMoviePlayerControllerAdBreakNotificationTypePreroll		@"preroll"
#define SASMoviePlayerControllerAdBreakNotificationTypeMidroll		@"midroll"
#define SASMoviePlayerControllerAdBreakNotificationTypePostroll		@"postroll"


@protocol SASMoviePlayerViewControllerDelegate;
@class SASPlayerAdConfig, SASPlayerDisplayConfig;


/** The `SASMoviePlayerController` class is used to manage the player view.
 
 An instance of this class must be instanciated. It manages the `SASMoviePlayerViewController` instance in order to
 display a content video and one or several ads during video playback.
 
 The `SASMoviePlayerController` instance can be instantiated with a contentURL and a `SASPlayerAdConfig` object. You can also pass a `SASPlayerDisplayConfig` instance.
 
 The notifications provided by the `SASMoviePlayerViewController` class are the following:
 
 - `SASMoviePlayerControllerAdBreakStartNotification`
 
 - `SASMoviePlayerControllerAdBreakEndNotification`
 
 - `SASMoviePlayerControllerDidEnterFullscreenNotification`
 
 - `SASMoviePlayerControllerDidExitFullscreenNotification`
 
 - `SASMoviePlayerControllerVideoSessionDidCompleteNotification`
 

 The ad break types (for `SASMoviePlayerControllerAdBreakStartNotification` & `SASMoviePlayerControllerAdBreakEndNotification` notifications)
 provided by the `SASMoviePlayerViewController` class are the following:

 - `SASMoviePlayerControllerAdBreakNotificationType`

 - `SASMoviePlayerControllerAdBreakNotificationTypePreroll`
 
 - `SASMoviePlayerControllerAdBreakNotificationTypeMidroll`
 
 - `SASMoviePlayerControllerAdBreakNotificationTypePostroll`

 */

@interface SASMoviePlayerViewController : NSObject <UIWebViewDelegate>


/** The receiver's delegate.
 
 The delegate must adopt the `SASMoviePlayerViewControllerDelegate` protocol.
 The delegate sends messages when the movie is playing. See `SASMoviePlayerViewControllerDelegate` Protocol Reference for the optional methods implemented in this delegate.

 @warning *Important* : Before releasing an instance of `SASMoviePlayerViewControllerDelegate` for which you have set a delegate, you must first set its delegate property to nil. This can be done, for example, in your dealloc method.
 */

@property (nonatomic, weak) id <SASMoviePlayerViewControllerDelegate> delegate;


/** The movie player controller object used to present the movie. (read-only)
 
 The `SASMoviePlayerController` object in this property is created automatically by the receiver and cannot be changed.
 However, you can use the object to manage the presentation and configuration of the movie playback or access properties.
 
 */

@property (nonatomic, strong, readonly) SASMoviePlayerController *moviePlayer;


/** The parent view controller of the `SASMoviePlayerViewController` object.
 
 The parent view controller displaying the modal view used to open the click URL of the ad.
 
 */

@property (nonatomic, weak) UIViewController *modalParentViewController;


/** The configuration object used to display ads.
  
 */

@property (nonatomic, strong) SASPlayerDisplayConfig *playerDisplayConfig;


/** Returns a `SASMoviePlayerViewController` object initialized with the specified movie URL and ad configuration.
 
 This method initializes a movie player, but does not prepare it for playback. To prepare a new movie player for playback, call the [`prepareToPlay`](https://developer.apple.com/library/ios/documentation/mediaplayer/reference/MPMediaPlayback_protocol/) method, described in MPMediaPlayback Protocol Reference.
 
 To be notified when a new movie player is ready to play, register for the `MPMoviePlayerLoadStateDidChangeNotification` notification. You can then check the loading state by accessing the loadState property.
 
 To check for errors during the URL loading, register for the `MPMoviePlayerPlaybackDidFinishNotification` notification. If an error occurs, this notification contains a `NSError` object available using the 'error' key in the notification's userInfo dictionary.
 
 @param contentURL The location of the movie file. This file must be located either in your app directory or on a remote server.
 @param adConfig The configuration used to get the ads.
 
 @return An initialized instance of `SASMoviePlayerViewController`.
 
 */

- (instancetype)initWithContentURL:(NSURL *)contentURL adConfig:(SASPlayerAdConfig *)adConfig;


/** Returns a `SASMoviePlayerViewController` object initialized with the specified movie URL, ad configuration and display configuration.
 
 This method initializes a movie player, but does not prepare it for playback. To prepare a new movie player for playback, call the `prepareToPlay` method, described in MPMediaPlayback Protocol Reference.
 
 To be notified when a new movie player is ready to play, register for the `MPMoviePlayerLoadStateDidChangeNotification` notification. You can then check the loading state by accessing the `loadState` property.
 
 To check for errors during the URL loading, register for the `MPMoviePlayerPlaybackDidFinishNotification` notification. If an error occurs, this notification contains a `NSError` object available using the 'error' key in the notification's userInfo dictionary.
 
 @param contentURL The location of the movie file. This file must be located either in your app directory or on a remote server.
 @param adConfig The configuration used to get the ads.
 @param displayConfig The configuration used for displaying the ads.
 
 @return An initialized instance of `SASMoviePlayerViewController`.
 
 */

- (instancetype)initWithContentURL:(NSURL *)contentURL adConfig:(SASPlayerAdConfig *)adConfig displayConfig:(SASPlayerDisplayConfig *)displayConfig;


/** Returns a `SASMoviePlayerViewController` object initialized with the specified movie URL, ad configuration display configuration and display mode.
 
 This method initializes a movie player, but does not prepare it for playback. To prepare a new movie player for playback, call the `prepareToPlay` method, described in MPMediaPlayback Protocol Reference.
 
 To be notified when a new movie player is ready to play, register for the `MPMoviePlayerLoadStateDidChangeNotification` notification. You can then check the loading state by accessing the `loadState` property.
 
 To check for errors during the URL loading, register for the `MPMoviePlayerPlaybackDidFinishNotification` notification. If an error occurs, this notification contains a `NSError` object available using the 'error' key in the notification's userInfo dictionary.
 
 @param contentURL The location of the movie file. This file must be located either in your app directory or on a remote server.
 @param adConfig The configuration used to get the ads.
 @param displayConfig The configuration used for displaying the ads.
 @param displayMode The mode that will be used for displaying controls. Valid values are:
	 
	 typedef enum {
		 SASMoviePlayerControlsDisplayModeDefault,
		 SASMoviePlayerControlsDisplayModeInline,
		 SASMoviePlayerControlsDisplayModeFullscreenOnly,
	 } SASMoviePlayerControlsDisplayMode;
 
 `SASMoviePlayerControlsDisplayModeDefault`
 
 Default controls layout. Display a fullscreen button and no navigation bar.
 
 `SASMoviePlayerControlsDisplayModeInline`
 
 Display no fullscreen button and no navigation bar.
 
 `SASMoviePlayerControlsDisplayModeFullscreenOnly`
 
 Display a navigation bar with a 'Done' button (that will send an exit fullscreen notification if clicked). Recommended display mode for modal video players.
 
 @return An initialized instance of `SASMoviePlayerViewController`.
 
 */

- (instancetype)initWithContentURL:(NSURL *)contentURL
						  adConfig:(SASPlayerAdConfig *)adConfig
					 displayConfig:(SASPlayerDisplayConfig *)displayConfig
					   displayMode:(SASMoviePlayerControlsDisplayMode)displayMode;


/** Returns a `SASMoviePlayerViewController` object initialized with the specified movie URL, ad configuration, display configuration, display mode and timeout.
 
 This method initializes a movie player, but does not prepare it for playback. To prepare a new movie player for playback, call the `prepareToPlay` method, described in MPMediaPlayback Protocol Reference.
 
 To be notified when a new movie player is ready to play, register for the `MPMoviePlayerLoadStateDidChangeNotification` notification. You can then check the loading state by accessing the `loadState` property.
 
 To check for errors during the URL loading, register for the `MPMoviePlayerPlaybackDidFinishNotification` notification. If an error occurs, this notification contains a `NSError` object available using the 'error' key in the notification's userInfo dictionary.
 
 @param contentURL The location of the movie file. This file must be located either in your app directory or on a remote server.
 @param adConfig The configuration used to get the ads.
 @param displayConfig The configuration used for displaying the ads.
 @param displayMode The mode that will be used for displaying controls. Valid values are:
 
	 typedef enum {
		 SASMoviePlayerControlsDisplayModeDefault,
		 SASMoviePlayerControlsDisplayModeInline,
		 SASMoviePlayerControlsDisplayModeFullscreenOnly,
	 } SASMoviePlayerControlsDisplayMode;
 
 `SASMoviePlayerControlsDisplayModeDefault`
 
 Default controls layout. Display a fullscreen button and no navigation bar.
 
 `SASMoviePlayerControlsDisplayModeInline`
 
 Display no fullscreen button and no navigation bar.
 
 `SASMoviePlayerControlsDisplayModeFullscreenOnly`
 
 Display a navigation bar with a 'Done' button (that will send an exit fullscreen notification if clicked). Recommended display mode for modal video players.
 
 @param adCallTimeout The timeout representing the time allowed to load the ad player entirely.

 @return An initialized instance of `SASMoviePlayerViewController`.
 
 */

- (instancetype)initWithContentURL:(NSURL *)contentURL
						  adConfig:(SASPlayerAdConfig *)adConfig
					 displayConfig:(SASPlayerDisplayConfig *)displayConfig
					   displayMode:(SASMoviePlayerControlsDisplayMode)displayMode
					 adCallTimeout:(NSTimeInterval)adCallTimeout;


/** Enables ad test mode.
 
 Calling this method will enable test mode and display default ads (prerolls, midrolls and postrolls) that always deliver.
 Enable test mode in your `application:didFinishLaunchingWithOptions:` method:
 
	- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
		[SASMoviePlayerViewController enableAdTestMode:YES];
	}
 
 */

+ (void)enableAdTestMode:(BOOL)adTestMode;

@end
